// MoI Nodeedit
// Widgets v.0.85 - Max Smirnov, 2016
// litegraph.js library (c) Javi Agenjo http://tamats.com

(function(){
	
	
// Widget Knob

function WidgetKnob()
{
	this.size = [55,58];
	this.minsize = [55,58];
	this.addOutput("",'numarray');
	this.local = { csize:50 }
	this.properties = { min:0, max:100, digits:0, value:"20" };
	this.internal = { position:0.2 };
}

WidgetKnob.title = "Knob";
WidgetKnob.desc = "Circular controller";

WidgetKnob.prototype.onAdded = function()
{
	this.internal.position = (this.properties.value - this.properties.min) / (this.properties.max - this.properties.min);
}

WidgetKnob.prototype.onPropertyChange = function()
{

	this.properties.value = parseFloat(this.properties.value.replace(',', '.'));
	if ( isNaN(this.properties.value) ) this.properties.value = this.properties.min;
	if ( this.properties.min >= this.properties.max ) this.properties.max = this.properties.min+1;
	if ( this.properties.value < this.properties.min ) this.properties.value = this.properties.min*1;
	if ( this.properties.value > this.properties.max ) this.properties.value =this.properties.max*1;
	this.properties.digits = Math.floor(this.properties.digits);
	if (this.properties.digits>12) this.properties.digits = 12;
	if (this.properties.digits<0) this.properties.digits = 0;
	this.internal.position = (this.properties.value-this.properties.min)/(this.properties.max-this.properties.min);
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
}

WidgetKnob.prototype.onDrawForeground = function(ctx)
{
	var cx = this.size[0]/2;
	var cy = this.size[1]/2;
	var ro = this.local.csize*0.45;
	var ri = this.local.csize*0.37;

	var bgr = this.bgcolor || LiteGraph.NODE_DEFAULT_BGCOLOR;
	ctx.fillStyle= bgr;
	ctx.beginPath();
		ctx.arc(cx, cy, this.local.csize*0.54, 0, Math.PI*2, false);
	ctx.fill();
	ctx.beginPath();
		ctx.strokeStyle= this.mouseOver ? "#FFF" : "#AAA";
		ctx.arc(this.size[0]/2, this.size[1]/2, this.local.csize*0.50, 0, 2*Math.PI, true);
	ctx.stroke();

	if(this.internal.position > 0)
	{
		ctx.beginPath();
			ctx.fillStyle="#9CA";
			ctx.moveTo(cx,cy-ro)
			ctx.arc(cx, cy, ro, 3/4*Math.PI*2, 3/4*Math.PI*2-Math.PI*2*this.internal.position, true);
			ctx.lineTo(cx,cy);
		ctx.fill();
		ctx.beginPath();
			ctx.fillStyle= bgr;
			ctx.arc(cx, cy, ri, 0, Math.PI*2, false);
		ctx.fill();

	}
	ctx.font = "12px Arial";
	ctx.fillStyle="#AAA";
	ctx.textAlign = "center";
	var dgt = parseInt(this.properties.digits);
	if ( dgt > 2 ) dgt = 2;
	if ( this.properties.max >= 100 && dgt > 1) dgt = 1;
	if ( this.properties.max >= 1000 ) dgt = 0;
	ctx.fillText(parseFloat(this.properties.value).toFixed(dgt), this.size[0]/2, this.size[1]/2+4 );
	ctx.textAlign = "left";
}

WidgetKnob.prototype.onExecute = function()
{
	this.setOutputData(0, [parseFloat(this.properties.value)] );
}

WidgetKnob.prototype.onMouseDown = function(e, skipcapture)
{
	this.center = [this.size[0]/2, this.size[1]/2];
	this.radius = this.size[0]/2;
	if(e.canvasY - this.pos[1] < 0 || distance([e.canvasX,e.canvasY],[this.pos[0] + this.center[0],this.pos[1] + this.center[1]]) > this.radius) return false;
	if ( skipcapture ) return true;
	this.oldmouse = [ e.canvasX - this.pos[0], e.canvasY - this.pos[1] ];
	this.captureInput(true);
	return true;
}

WidgetKnob.prototype.onMouseMove = function(e)
{
	if (!this.oldmouse) return;

	var m = [ e.canvasX - this.pos[0], e.canvasY - this.pos[1] ];

	var v = this.internal.position;
	var delta = (m[1] - this.oldmouse[1]);

	v -= delta/201;
	if (v > 1.0) { v = 1.0 } else if (v < 0.0) { v = 0.0 }

	this.internal.position = v;
	this.properties.value = this.properties.min + (this.properties.max - this.properties.min) * this.internal.position;
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
	this.oldmouse = m;
	this.updateThisNodeGraph();
	this.graph.setisChangedFlag(this.id);
}

WidgetKnob.prototype.onMouseUp = function()
{
	if(this.oldmouse)
	{
		this.oldmouse = null;
		this.captureInput(false);
	}
}

LiteGraph.registerNodeType("Widget/Knob", WidgetKnob);


// Widget Slider

function WidgetHSlider()
{
	this.size = [150,20];
	this.minsize = [150,20];
	this.addOutput("",'numarray');
	this.local = { shiftLeft:14, shiftRight:40, fontsize:12 };
	this.properties = { min:0, max:10, digits:1, value:"5.0" };
	this.internal = { position:0.5 };
}

WidgetHSlider.title = "Slider";
WidgetHSlider.desc = "Linear slider controller";

WidgetHSlider.prototype.onAdded = function()
{
	this.internal.position = (this.properties.value - this.properties.min) / (this.properties.max - this.properties.min);
}

WidgetHSlider.prototype.onPropertyChange = function()
{
	this.properties.value = parseFloat(this.properties.value.replace(',', '.'));
	if ( isNaN(this.properties.value) ) this.properties.value = this.properties.min;
	if ( this.properties.min >= this.properties.max ) this.properties.max = this.properties.min+1;
	if ( this.properties.value < this.properties.min ) this.properties.value = this.properties.min;
	if ( this.properties.value > this.properties.max ) this.properties.value = this.properties.max;
	this.properties.digits = Math.floor(this.properties.digits);
	if (this.properties.digits>12) this.properties.digits = 12;
	if (this.properties.digits<0) this.properties.digits = 0;
	this.internal.position = (this.properties.value-this.properties.min)/(this.properties.max-this.properties.min);
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
}

WidgetHSlider.prototype.onDrawForeground = function(ctx)
{
	var sr = this.local.shiftRight, sl = this.local.shiftLeft;
	var dgt = parseInt(this.properties.digits);
	if ( dgt > 2 ) dgt = 2;
	if ( this.properties.max >= 100 && dgt > 1) dgt = 1;
	if ( this.properties.max >= 1000 ) dgt = 0;

//	ctx.lineWidth = 1;
	ctx.fillStyle=this.bgcolor || LiteGraph.NODE_DEFAULT_BGCOLOR;
	ctx.fillRect( 3, 2, this.size[0]-6.5, 16);
	
	ctx.fillStyle="#222";
	ctx.fillRect( sl, 8, this.size[0]-sr-sl, 4);
	ctx.beginPath();
		ctx.arc(this.size[0]-sr, 10, 2, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#888";
	ctx.fillRect( sl, 8, (this.size[0]-sr-sl)*this.internal.position, 4);
	ctx.beginPath();
		ctx.arc(sl, 10, 2, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#AAA";
	ctx.beginPath();
		ctx.arc(sl+(this.size[0]-sr-sl)*this.internal.position, 10, 6, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#AAA";
	ctx.font = (this.local.fontsize) + "px Arial";
	ctx.textAlign = "center";
	ctx.fillText(parseFloat(this.properties.value).toFixed(dgt), this.size[0]-sr+20, (10+this.local.fontsize/3));

}

WidgetHSlider.prototype.onExecute = function()
{
	this.setOutputData(0, [parseFloat(this.properties.value)] );
}

WidgetHSlider.prototype.onMouseDown = function(e)
{
	if ( e.canvasX < this.pos[0]+this.local.shiftLeft-5 || e.canvasX > this.pos[0]+this.size[0]-this.local.shiftRight+5 ) return false;
	if (e.canvasY - this.pos[1] < 0) return false;
	this.capture = true;
	this.captureInput(true);
	this.onMouseMove(e);
	return true;
}

WidgetHSlider.prototype.onMouseMove = function(e)
{
	if (!this.capture) return;
	var v = (e.canvasX - this.pos[0] - this.local.shiftLeft)/(this.size[0]-this.local.shiftRight-this.local.shiftLeft);
	if ( v < 0 ) { v = 0 } else if ( v > 1 ) { v = 1 }
	this.internal.position = v;
	this.properties.value = this.properties.min + (this.properties.max - this.properties.min) * this.internal.position;
	this.properties.value = this.properties.value.toFixed(this.properties.digits);
	this.updateThisNodeGraph();
	this.graph.setisChangedFlag(this.id);
}

WidgetHSlider.prototype.onMouseUp = function()
{
	this.capture = false;
	this.captureInput(false);
}

LiteGraph.registerNodeType("Widget/Slider", WidgetHSlider );

// Widget RngSlider

function WidgetRngSlider()
{
	this.size = [200,20];
	this.minsize = [200,20];
	this.addOutput("",'numarray');
	this.local = { shiftLeft:14, shiftRight:80, fontsize:12 };
	this.properties = { min:0, max:10, digits:1, from:"0.0", to:"10.0" };
	this.internal = { positionF:0.0, positionT:1.0 };
}

WidgetRngSlider.title = "Range";
WidgetRngSlider.desc = "Linear range controller";

WidgetRngSlider.prototype.onAdded = function()
{
	this.internal.positionF = (this.properties.from - this.properties.min) / (this.properties.max - this.properties.min);
	this.internal.positionT = (this.properties.to - this.properties.min) / (this.properties.max - this.properties.min);
}

WidgetRngSlider.prototype.onPropertyChange = function(prop)
{
	this.properties.from = parseFloat(this.properties.from.replace(',', '.'));
	this.properties.to = parseFloat(this.properties.to.replace(',', '.'));
	if ( isNaN(this.properties.from) ) this.properties.from = this.properties.min;
	if ( isNaN(this.properties.to) ) this.properties.to = this.properties.max;

	this.properties.digits = Math.floor(this.properties.digits);
	if (this.properties.digits>15) this.properties.digits = 15;
	if (this.properties.digits<0) this.properties.digits = 0;
	var s = 1/Math.pow(10,this.properties.digits);

	this.properties.from = this.properties.from.toFixed(this.properties.digits)*1.0;
	this.properties.to = this.properties.to.toFixed(this.properties.digits)*1.0;
	this.properties.max = this.properties.max.toFixed(this.properties.digits)*1.0;
	this.properties.min = this.properties.min.toFixed(this.properties.digits)*1.0;
	
	if (prop === "max" || prop === "digits")
	{
		if ( this.properties.max <= this.properties.min ) this.properties.max = this.properties.min+s;
	}

	if (prop === "min" || prop === "digits")
	{
		if ( this.properties.min >= this.properties.max ) this.properties.min = this.properties.max-s;
	}

	if (prop === "to" || prop === "max" || prop === "digits")
	{
		if ( this.properties.to <= this.properties.from ) this.properties.to = this.properties.from+s;
		if ( this.properties.to > this.properties.max ) this.properties.to = this.properties.max;
		if ( this.properties.from >= this.properties.to ) this.properties.from = this.properties.to-s;
	}

	if (prop === "from" || prop === "min" || prop === "digits")
	{
		if ( this.properties.from >= this.properties.to ) this.properties.from = this.properties.to-s;
		if ( this.properties.from < this.properties.min ) this.properties.from = this.properties.min;
		if ( this.properties.to <= this.properties.from ) this.properties.to = this.properties.from+s;
	}

	this.internal.positionF = (this.properties.from-this.properties.min)/(this.properties.max-this.properties.min);
	this.internal.positionT = (this.properties.to-this.properties.min)/(this.properties.max-this.properties.min);

	this.properties.from = this.properties.from.toFixed(this.properties.digits);
	this.properties.to = this.properties.to.toFixed(this.properties.digits);
}

WidgetRngSlider.prototype.onDrawForeground = function(ctx)
{
	var sr = this.local.shiftRight, sl = this.local.shiftLeft, sf = sl+(this.size[0]-sr-sl)*this.internal.positionF, st = sl+(this.size[0]-sr-sl)*this.internal.positionT;
	var dgt = parseInt(this.properties.digits);
	if ( dgt > 2 ) dgt = 2;
	if ( this.properties.max >= 100 && dgt > 1) dgt = 1;
	if ( this.properties.max >= 1000 ) dgt = 0;

//	ctx.lineWidth = 1;
	ctx.fillStyle=this.bgcolor || LiteGraph.NODE_DEFAULT_BGCOLOR;
	ctx.fillRect( 3, 2, this.size[0]-6.5, 16);
	
	ctx.fillStyle="#222";
	ctx.fillRect( sl, 8, this.size[0]-sr-sl, 4);
	ctx.beginPath();
		ctx.arc(this.size[0]-sr, 10, 2, 0, 2 * Math.PI, false);
	ctx.fill();
	ctx.beginPath();
		ctx.arc(sl, 10, 2, 0, 2 * Math.PI, false);
	ctx.fill();

	ctx.fillStyle="#888";
	ctx.fillRect( sl+(this.size[0]-sr-sl)*this.internal.positionF, 8, (this.size[0]-sr-sl)*this.internal.positionT-(this.size[0]-sr-sl)*this.internal.positionF, 4);

	ctx.fillStyle="#AAA";
	ctx.beginPath();
		ctx.moveTo(sf,18);
		ctx.lineTo(sf,2);
		ctx.lineTo(sf-6,6);
		ctx.lineTo(sf-6,14);
	ctx.fill();
	ctx.beginPath();
		ctx.moveTo(st,18);
		ctx.lineTo(st,2);
		ctx.lineTo(st+6,6);
		ctx.lineTo(st+6,14);
	ctx.fill();

	ctx.fillStyle="#AAA";
	ctx.font = (this.local.fontsize) + "px Arial";
	ctx.textAlign = "center";
	ctx.fillText(parseFloat(this.properties.from).toFixed(dgt)+".."+parseFloat(this.properties.to).toFixed(dgt), this.size[0]-sr+40, (10+this.local.fontsize/3));

}

WidgetRngSlider.prototype.onExecute = function()
{
	this.setOutputData(0, [parseFloat(this.properties.from),parseFloat(this.properties.to)] );
}

WidgetRngSlider.prototype.onMouseDown = function(e)
{
	if ( e.canvasX < this.pos[0]+this.local.shiftLeft-5 || e.canvasX > this.pos[0]+this.size[0]-this.local.shiftRight+5 ) return false;
	if (e.canvasY - this.pos[1] < 0) return false;
	var v = (e.canvasX-this.pos[0]-this.local.shiftLeft)/(this.size[0]-this.local.shiftLeft-this.local.shiftRight);
	this.movefrom = ( Math.abs(v-this.internal.positionF) < Math.abs(v-this.internal.positionT) )?true:false;
	this.moveto = !this.movefrom;
	this.capture = true;
	this.captureInput(true);
	this.onMouseMove(e);
	return true;
}

WidgetRngSlider.prototype.onMouseMove = function(e)
{
	if (!this.capture) return;
	var v = (e.canvasX - this.pos[0] - this.local.shiftLeft)/(this.size[0]-this.local.shiftRight-this.local.shiftLeft);
	var s = Math.pow(0.1,this.properties.digits)/(this.properties.max - this.properties.min);
	if (this.movefrom)
	{
		if ( v < 0 ) { v = 0 } else if ( v > this.internal.positionT-s ) { v = this.internal.positionT-s }
		this.internal.positionF = v;
		this.properties.from = this.properties.min + (this.properties.max - this.properties.min) * this.internal.positionF;
		this.properties.from = this.properties.from.toFixed(this.properties.digits);
	}
	if (this.moveto)
	{
		if ( v > 1 ) { v = 1 } else if ( v < this.internal.positionF+s ) { v = this.internal.positionF+s }
		this.internal.positionT = v;
		this.properties.to = this.properties.min + (this.properties.max - this.properties.min) * this.internal.positionT;
		this.properties.to = this.properties.to.toFixed(this.properties.digits);
	}
	this.updateThisNodeGraph();
	this.graph.setisChangedFlag(this.id);
}

WidgetRngSlider.prototype.onMouseUp = function()
{
	this.capture = false;
	this.movefrom = false;
	this.moveto = false;
	this.captureInput(false);
}

LiteGraph.registerNodeType("Widget/RngSlider", WidgetRngSlider );

})();